#
# OpenVMP, 2023
#
# Author: Roman Kuzmenko
# Created: 2023-12-22
#
# Licensed under Apache License, Version 2.0.
#

DEFAULT_RENDER_WIDTH = 1024
DEFAULT_RENDER_HEIGHT = 1024

DEFAULT_RENDER_SVG_OPTS = {
    "width": DEFAULT_RENDER_WIDTH,
    "height": DEFAULT_RENDER_HEIGHT,
    "showAxes": False,
    "projectionDir": [1, -1, 0.75],
    "strokeWidth": 0.25,
    "strokeColor": [64, 192, 64],
    "hiddenColor": [32, 64, 32],
    "showHidden": False,
    # "marginLeft": 0,
    # "marginTop": 0,
}
